"use strict";
// Copyright IBM Corp. 2020. All Rights Reserved.
// Node module: @loopback/example-socketio
// This file is licensed under the MIT License.
// License text available at https://opensource.org/licenses/MIT
Object.defineProperty(exports, "__esModule", { value: true });
exports.SocketIoController = void 0;
const tslib_1 = require("tslib");
const socketio_1 = require("@codeloop/socketio");
const debug_1 = tslib_1.__importDefault(require("debug"));
const debug = debug_1.default('loopback:socketio:controller');
/**
 * A demo controller for socket.io
 *
 * ```ts
 * @socketio('/')
 * ```
 * This specifies the namespace / for this controller
 * Regex or strings are acceptable values for namespace
 */
let SocketIoController = class SocketIoController {
    constructor(socket) {
        this.socket = socket;
    }
    /**
     * The method is invoked when a client connects to the server
     *
     * @param socket - The socket object for client
     */
    async connect(socket) {
        debug('Client connected: %s', this.socket.id);
        await socket.join('room 1');
    }
    /**
     * Register a handler for 'subscribe-to-channel' events
     *
     * @param msg - The message sent by client
     */
    // @socketio.emit('namespace' | 'requestor' | 'broadcast')
    async registerChannel(msg) {
        debug('Subscribe to channel: %s', msg);
        if (Array.isArray(msg) && msg.length > 0) {
            await Promise.all(msg.map(async (item) => {
                await this.socket.join(item);
            }));
        }
        else {
            throw new Error('Channels data not appropriate');
        }
    }
    /**
     * Register a handler for 'general-message-forward' events
     *
     * @param msg - The message sent by client
     */
    // @socketio.emit('namespace' | 'requestor' | 'broadcast')
    handleChatMessage(msg) {
        debug('General forwarded message: %s', msg);
        this.socket.nsp.emit('general-message-forward', msg);
    }
    /**
     * Register a handler for 'general-message' events
     *
     * @param msg - The message sent by client
     */
    // @socketio.emit('namespace' | 'requestor' | 'broadcast')
    handleGeneralMessage(msg) {
        var _a, _b;
        debug('General Message : %s', msg);
        const parsedMsg = JSON.parse(msg);
        if (((_b = (_a = parsedMsg === null || parsedMsg === void 0 ? void 0 : parsedMsg.receiver) === null || _a === void 0 ? void 0 : _a.to) === null || _b === void 0 ? void 0 : _b.length) > 0) {
            parsedMsg.receiver.to.forEach(item => this.socket.nsp.to(item.id).emit('message', {
                subject: parsedMsg.subject,
                body: parsedMsg.body,
                options: parsedMsg.options,
            }));
        }
        else {
            throw new Error('Inappropriate message data');
        }
    }
    /**
     * Register a handler for all events
     */
    logMessage(...args) {
        debug('Message: %s', args);
    }
    /**
     * The method is invoked when a client disconnects from the server
     * @param socket
     */
    disconnect() {
        debug('Client disconnected: %s', this.socket.id);
    }
};
tslib_1.__decorate([
    socketio_1.socketio.connect(),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [socketio_1.Socket]),
    tslib_1.__metadata("design:returntype", Promise)
], SocketIoController.prototype, "connect", null);
tslib_1.__decorate([
    socketio_1.socketio.subscribe('subscribe-to-channel'),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [Array]),
    tslib_1.__metadata("design:returntype", Promise)
], SocketIoController.prototype, "registerChannel", null);
tslib_1.__decorate([
    socketio_1.socketio.subscribe('general-message-forward'),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [Object]),
    tslib_1.__metadata("design:returntype", void 0)
], SocketIoController.prototype, "handleChatMessage", null);
tslib_1.__decorate([
    socketio_1.socketio.subscribe('general-message'),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [String]),
    tslib_1.__metadata("design:returntype", void 0)
], SocketIoController.prototype, "handleGeneralMessage", null);
tslib_1.__decorate([
    socketio_1.socketio.subscribe(/.+/),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [Object]),
    tslib_1.__metadata("design:returntype", void 0)
], SocketIoController.prototype, "logMessage", null);
tslib_1.__decorate([
    socketio_1.socketio.disconnect(),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", []),
    tslib_1.__metadata("design:returntype", void 0)
], SocketIoController.prototype, "disconnect", null);
SocketIoController = tslib_1.__decorate([
    socketio_1.socketio('/'),
    tslib_1.__param(0, socketio_1.socketio.socket()),
    tslib_1.__metadata("design:paramtypes", [socketio_1.Socket])
], SocketIoController);
exports.SocketIoController = SocketIoController;
//# sourceMappingURL=socketio.controller.js.map